@@ -3,6 +3,8 @@ import pyro.distributions as dist
 from pyro.optim import ExponentialLR
 from pyro.infer import SVI, JitTraceEnum_ELBO, TraceEnum_ELBO, config_enumerate
 
+import scvi.distributions as scvi_dist 
+
 import torch
 import torch.nn as nn
 from torch.utils.data import DataLoader
@@ -389,22 +391,27 @@ class DensityFlow(nn.Module):
             log_mu = self.decoder_log_mu(zs)
             if self.loss_func in ['bernoulli']:
                 log_theta = log_mu
-            elif self.loss_func == 'negbinomial':
-                mu = log_mu.exp()
+            #elif self.loss_func == 'negbinomial':
+            #    mu = log_mu.exp()
             else:
                 rate = log_mu.exp()
                 theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
                 if self.loss_func == 'poisson':
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
+                elif self.loss_func == 'negbinomial':
+                    mu = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
-                #logits = (dispersion.log()-mu.log()).clamp(min=-15, max=15)
                 if self.use_zeroinflate:
-                    pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, 
-                                                                                         logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.ZeroInflatedNegativeBinomial(mu=mu, theta=theta, zi_logits=gate_logits)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
-                    pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.NegativeBinomial(mu=mu, theta=theta)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
             elif self.loss_func == 'poisson':
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.Poisson(rate=rate),gate_logits=gate_logits).to_event(1), obs=xs.round())
@@ -478,21 +485,27 @@ class DensityFlow(nn.Module):
             log_mu = self.decoder_log_mu(zs)
             if self.loss_func in ['bernoulli']:
                 log_theta = log_mu
-            elif self.loss_func == 'negbinomial':
-                mu = log_mu.exp()
+            #elif self.loss_func == 'negbinomial':
+            #    mu = log_mu.exp()
             else:
                 rate = log_mu.exp()
                 theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
                 if self.loss_func == 'poisson':
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
+                elif self.loss_func == 'negbinomial':
+                    mu = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
-                #logits = (dispersion.log()-mu.log()).clamp(min=-15, max=15)
                 if self.use_zeroinflate:
-                    pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.ZeroInflatedNegativeBinomial(mu=mu, theta=theta, zi_logits=gate_logits)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
-                    pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.NegativeBinomial(mu=mu, theta=theta)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
             elif self.loss_func == 'poisson':
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.Poisson(rate=rate),gate_logits=gate_logits).to_event(1), obs=xs.round())
@@ -578,22 +591,27 @@ class DensityFlow(nn.Module):
             log_mu = self.decoder_log_mu(zs)
             if self.loss_func in ['bernoulli']:
                 log_theta = log_mu
-            elif self.loss_func in ['negbinomial']:
-                mu = log_mu.exp()
+            #elif self.loss_func in ['negbinomial']:
+            #    mu = log_mu.exp()
             else:
                 rate = log_mu.exp()
                 theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
                 if self.loss_func == 'poisson':
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
+                elif self.loss_func == 'negbinomial':
+                    mu = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
-                #logits = (dispersion.log()-mu.log()).clamp(min=-15, max=15)
                 if self.use_zeroinflate:
-                    pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, 
-                                                                                         logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.ZeroInflatedNegativeBinomial(mu=mu, theta=theta, zi_logits=gate_logits)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
-                    pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.NegativeBinomial(mu=mu, theta=theta)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
             elif self.loss_func == 'poisson':
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.Poisson(rate=rate),gate_logits=gate_logits).to_event(1), obs=xs.round())
@@ -689,22 +707,27 @@ class DensityFlow(nn.Module):
             log_mu = self.decoder_log_mu(zs)
             if self.loss_func in ['bernoulli']:
                 log_theta = log_mu
-            elif self.loss_func in ['negbinomial']:
-                mu = log_mu.exp()
+            #elif self.loss_func in ['negbinomial']:
+            #    mu = log_mu.exp()
             else:
                 rate = log_mu.exp()
                 theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
                 if self.loss_func == 'poisson':
                     rate = theta * torch.sum(xs, dim=1, keepdim=True)
+                elif self.loss_func == 'negbinomial':
+                    mu = theta * torch.sum(xs, dim=1, keepdim=True)
 
             if self.loss_func == 'negbinomial':
-                logits = (mu.log()-dispersion.log()).clamp(min=-15, max=15)
-                #logits = (dispersion.log()-mu.log()).clamp(min=-15, max=15)
                 if self.use_zeroinflate:
-                    pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, 
-                                                                                         logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.ZeroInflatedNegativeBinomial(mu=mu, theta=theta, zi_logits=gate_logits)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.ZeroInflatedDistribution(dist.NegativeBinomial(total_count=dispersion, logits=logits),gate_logits=gate_logits).to_event(1), obs=xs)
                 else:
-                    pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
+                    base_dist = scvi_dist.NegativeBinomial(mu=mu, theta=theta)
+                    event_dist = dist.Independent(base_dist, 1)
+                    pyro.sample("x", event_dist, obs=xs)
+                    #pyro.sample('x', dist.NegativeBinomial(total_count=dispersion, logits=logits).to_event(1), obs=xs)
             elif self.loss_func == 'poisson':
                 if self.use_zeroinflate:
                     pyro.sample('x', dist.ZeroInflatedDistribution(dist.Poisson(rate=rate),gate_logits=gate_logits).to_event(1), obs=xs.round())
